@@ -2,7 +2,7 @@
 /*! @file inkf
     @brief yet another nkf with international language support
     @author Tachibanamasashi
-    @date 2002-2009
+    @date 2002-2011
  ******************************************************************************/
 
 #include <stdio.h>
@@ -11,16 +11,14 @@
 #include <unistd.h>
 
 #include "apolloron.h"
-#include "apolloron_http_client.h"
 
 using namespace apolloron;
-using namespace apolloron_http_client;
 
 
 const char *INKF_PROGNAME = "inkf";
 const char *INKF_BRIEF    = "Yet Another Network Kanji Filter";
-const char *INKF_VERSION  = "1.0.7";
-const char *INKF_COPY     = "Copyright (C) 2002-2009 Tachibanamasashi";
+const char *INKF_VERSION  = "1.0.8";
+const char *INKF_COPY     = "Copyright (C) 2002-2011 Tachibanamasashi";
 const char *INKF_DEF_OUT  = "inkf.out";
 
 typedef enum {
