@@ -19,7 +19,7 @@ namespace apolloron {
  *  @param void
  *  @return void
  */
-CGI::CGI(int argc, const char *argv[]) {
+CGI::CGI() {
   String str;
   char buf[1024 + 1];
   const char *cpstr;
@@ -302,6 +302,7 @@ CGI::CGI(int argc, const char *argv[]) {
       char *p, *prev_key, *prev_value;
       String tmp_key, esc_tmp_key;
 
+      str.useAsText();
       p = new char [len + 1];
       strcpy(p, str.c_str());
       prev_key = p;
@@ -337,8 +338,13 @@ CGI::CGI(int argc, const char *argv[]) {
     long length;
     char *p;
     p_base = cpstr;
-    while ((p_next = strchr(p_base, ';')) != NULL) {
-      length = p_next - p_base;
+    while (p_base[0] != '\0') {
+      p_next = strchr(p_base, ';');
+      if (p_next) {
+        length = p_next - p_base;
+      } else {
+        length = strlen(p_base);
+      }
       if (0 < length) {
         pstr = new char [length + 1];
         strncpy(pstr, p_base, length);
@@ -349,13 +355,14 @@ CGI::CGI(int argc, const char *argv[]) {
           key = (const char *)pstr;
           value = (const char *)(p + 1);
           if (key[0] != '\0') {
-            str = value;
-            (*this).cookie[key] = str.unescapeHTML();
+            String s = value;
+            (*this).cookie[key] = s.unescapeHTML();
+            s.clear();
           }
         }
         delete [] pstr;
       }
-      p_base = p_next + 1;
+      p_base = (p_next)?(p_next + 1):(p_base + strlen(p_base));
     }
   }
 
@@ -378,51 +385,155 @@ CGI::~CGI() {
  *  @retval false  failure
  */
 bool CGI::clear() {
-  return (*this).requestData.clear() && (*this).requestFileName.clear() && (*this).cookie.clear();
+  (*this).putHead = false;
+  return (*this).requestData.clear() && (*this).requestFileName.clear() &&
+      (*this).cookie.clear() && (*this).header.clear();
 }
 
 
 /*! Delete instance of CGI.
- *  @param key
+ *  @param  key  key of CGI parameter
  *  @return value of key
  */
 String& CGI::getValue(const String &key) {
-  return  (*this).requestData[key];
+  return (*this).requestData[key];
 }
 
 
 /*! Delete instance of CGI.
- *  @param key
+ *  @param  key  key of CGI parameter
  *  @return value of key
  */
 String& CGI::getValue(const char *key) {
-  return  (*this).requestData[key];
+  return (*this).requestData[key];
+}
+
+
+/*! Get environment variable's value
+ *  @param  key  key of Environment variable
+ *  @return value of key
+ */
+const char *CGI::getEnv(const String &key) const {
+  return getenv(key.c_str());
+}
+
+
+/*! Get environment variable's value
+ *  @param  key  key of Environment variable
+ *  @return value of key
+ */
+const char *CGI::getEnv(const char *key) const {
+  return getenv(key?key:"");
 }
 
 
 /*! Delete instance of CGI.
- *  @param key
+ *  @param  key  key of CGI parameter
  *  @return value of key
  */
 String& CGI::getFileName(const String &key, const char * src_charset, const char * dest_charset) {
-  return  ((String &)((*this).requestFileName[key])).unescapeQuote(src_charset, dest_charset);
+  return ((String &)((*this).requestFileName[key])).unescapeQuote(src_charset, dest_charset);
 }
 
 
 /*! Delete instance of CGI.
- *  @param key
+ *  @param  key  key of CGI parameter
  *  @return value of key
  */
 String& CGI::getFileName(const char *key, const char * src_charset, const char * dest_charset) {
-  return  ((String &)((*this).requestFileName[key])).unescapeQuote(src_charset, dest_charset);
+  return ((String &)((*this).requestFileName[key])).unescapeQuote(src_charset, dest_charset);
 }
 
-/*! Delete instance of CGI.
- *  @param key
+/*! Get cookie valiables.
+ *  @param  key  key of Cookie
  *  @return value of key
  */
 String& CGI::getCookie(const char *key, const char * src_charset, const char * dest_charset) {
-  return  ((String &)((*this).cookie[key])).strconv(src_charset, dest_charset);
+  return ((String &)((*this).cookie[key])).strconv(src_charset, dest_charset);
+}
+
+
+/*! Append HTTP header line
+ *  @param  key   key of HTTP header
+ *  @param  value value of key of HTTP header
+ *  @retval true  success
+ *  @retval false failure
+ */
+bool CGI::setHeader(const String &key, const String &value) {
+  return (*this).header.add(key + ": " + value);
+}
+
+
+/*! Clear HTTP header lines
+ *  @param void
+ *  @retval true  success
+ *  @retval false failure
+ */
+bool CGI::clearHeader() {
+  return (*this).header.clear();
+}
+
+
+/*! Put HTTP header lines (put "\r\n" footer)
+ *  @param void
+ *  @retval true  success
+ *  @retval false failure
+ */
+bool CGI::putHeader() {
+  long i, max;
+
+  if ((*this).putHead == true) {
+    return true;
+  }
+
+  max = header.max();
+
+  for (i = 0; i < max; i++) {
+    fputs(header[i].c_str(), stdout);
+    fputs("\r\n", stdout);
+  }
+
+  fputs("\r\n", stdout);
+  (*this).putHead = true;
+
+  return true;
+}
+
+
+/*! Put HTTP content (put header if not put)
+ *  @param content output content
+ *  @param size    size of output content
+ *  @retval true  success
+ *  @retval false failure
+ */
+bool CGI::putContent(const String &content, long size) {
+  if (size < 0) {
+    size = content.len();
+  }
+
+  return (*this).putContent(content.c_str(), size);
+}
+
+
+/*! Put HTTP content (put header if not put)
+ *  @param content output content
+ *  @param size    size of output content
+ *  @retval true  success
+ *  @retval false failure
+ */
+bool CGI::putContent(const char *content, long size) {
+  if (content == NULL) {
+    content = "";
+  }
+  if (size < 0) {
+    size = strlen(content);
+  }
+
+  (*this).putHeader();
+
+  fwrite(content, (size_t)size, 1, stdout);
+
+  return true;
 }
 
 
