@@ -10,6 +10,7 @@
 #include <time.h>
 #include <sys/time.h>
 
+#include "calendar/msg.h"
 #include "apolloron.h"
 
 namespace {
@@ -328,8 +329,12 @@ namespace apolloron {
     @return void
  */
 DateTime::DateTime() {
-  (*this).tmpString = new String;
-  clear();
+  (*this).pTmp = new (String **);
+  *((*this).pTmp) = NULL;
+  (*this).pTmpLen = new long;
+  *((*this).pTmpLen) = 0;
+
+  (*this).clear();
 }
 
 
@@ -338,9 +343,13 @@ DateTime::DateTime() {
     @return void
  */
 DateTime::DateTime(DateTime &datetime) {
-  (*this).tmpString = new String;
-  clear();
-  set(datetime);
+  (*this).pTmp = new (String **);
+  *((*this).pTmp) = NULL;
+  (*this).pTmpLen = new long;
+  *((*this).pTmpLen) = 0;
+
+  (*this).clear();
+  (*this).set(datetime);
 }
 
 
@@ -349,9 +358,13 @@ DateTime::DateTime(DateTime &datetime) {
     @return void
  */
 DateTime::DateTime(TDateTime &datetime) {
-  (*this).tmpString = new String;
-  clear();
-  set(datetime);
+  (*this).pTmp = new (String **);
+  *((*this).pTmp) = NULL;
+  (*this).pTmpLen = new long;
+  *((*this).pTmpLen) = 0;
+
+  (*this).clear();
+  (*this).set(datetime);
 }
 
 
@@ -361,10 +374,13 @@ DateTime::DateTime(TDateTime &datetime) {
     @return void
  */
 DateTime::DateTime(time_t time_t_time, long gmtoff) {
-  (*this).tmpString = new String;
-  clear();
+  (*this).pTmp = new (String **);
+  *((*this).pTmp) = NULL;
+  (*this).pTmpLen = new long;
+  *((*this).pTmpLen) = 0;
 
-  set(time_t_time, gmtoff);
+  (*this).clear();
+  (*this).set(time_t_time, gmtoff);
 }
 
 
@@ -377,10 +393,13 @@ DateTime::DateTime(time_t time_t_time, long gmtoff) {
     @return void
  */
 DateTime::DateTime(String &datetimestr) {
-  (*this).tmpString = new String;
-  clear();
+  (*this).pTmp = new (String **);
+  *((*this).pTmp) = NULL;
+  (*this).pTmpLen = new long;
+  *((*this).pTmpLen) = 0;
 
-  set(datetimestr);
+  (*this).clear();
+  (*this).set(datetimestr);
 }
 
 
@@ -389,11 +408,69 @@ DateTime::DateTime(String &datetimestr) {
     @return void
  */
 DateTime::~DateTime() {
-  //clear();
-  if ((*this).tmpString != (String *)NULL) {
-    delete (*this).tmpString;
-    (*this).tmpString = (String *)NULL;
+  clear();
+
+  if ((*this).pTmp != NULL) {
+    delete [] (*this).pTmp;
+    (*this).pTmp = NULL;
   }
+  if ((*this).pTmpLen != NULL) {
+    delete (*this).pTmpLen;
+    (*this).pTmpLen = NULL;
+  }
+}
+
+
+/*! Acquisition of a temporary String
+    @param void
+    @return Pointer of a temporary String.
+ */
+String* DateTime::tmpStr() const {
+  if (*pTmp == NULL) {
+    *((*this).pTmp) = new (String *);
+    **((*this).pTmp) = NULL;
+  }
+  if (**pTmp != NULL) {
+    String **ptmptmp;
+    ptmptmp = new String * [(*((*this).pTmpLen)) + 1];
+    memcpy(ptmptmp, *((*this).pTmp), sizeof(String *) * (*((*this).pTmpLen)));
+    delete [] *((*this).pTmp);
+    *((*this).pTmp) = ptmptmp;
+    (*((*this).pTmp))[*((*this).pTmpLen)] = new String;
+    (*((*this).pTmpLen))++;
+  } else {
+    **pTmp = new String;
+    *((*this).pTmpLen) = 1;
+  }
+  return **((*this).pTmp);
+}
+
+
+/*! Delete temporary Strings currently held
+    @param void
+    @retval true  success
+    @retval false failure
+ */
+bool DateTime::clearTmpStr() const {
+  if ((*this).pTmp != NULL) {
+    if (*((*this).pTmp) != NULL) {
+      if (**((*this).pTmp) != NULL) {
+        long i;
+        for (i = 0; i < *((*this).pTmpLen) ; i++) {
+          if ((*((*this).pTmp))[i] != NULL) {
+            delete ((*((*this).pTmp))[i]);
+          }
+        }
+        **pTmp = NULL;
+      }
+      delete [] *((*this).pTmp);
+      *((*this).pTmp) = NULL;
+    }
+  }
+  if ((*this).pTmpLen != NULL) {
+    *((*this).pTmpLen) = 0;
+  }
+  return true;
 }
 
 
@@ -412,7 +489,7 @@ bool DateTime::clear() {
   (*this).tmpDateTime.date.wday = 4;
   (*this).tmpDateTime.gmtoff = _get_server_gmtoff();
 
-  *((*this).tmpString) = "";
+  clearTmpStr();
 
   return true;
 }
@@ -423,7 +500,7 @@ bool DateTime::clear() {
     @return void
  */
 DateTime &DateTime::operator = (const DateTime &datetime) {
-  set(datetime);
+  (*this).set(datetime);
   return (*this);
 }
 
@@ -439,7 +516,7 @@ bool DateTime::setNow() {
 
   gmtoff = _get_server_gmtoff();
   tmp_time_t = time(NULL);
-  //tmp_time_t += gmtoff;
+  tmp_time_t += gmtoff;
 
   set(tmp_time_t, gmtoff);
 
@@ -863,7 +940,8 @@ bool DateTime::setGMTOffset(long gmtoff) {
              "%%"  :  %
     @return formatted date and/or time String
  */
-String &DateTime::toString(const String &format) {
+String &DateTime::toString(const String &format) const {
+  String *tmp = (*this).tmpStr();
   const char *cformat;
   char buf[100];
   long i, len, gmtoff;
@@ -871,125 +949,283 @@ String &DateTime::toString(const String &format) {
   cformat = format.c_str();
   len = format.len();
 
-  *((*this).tmpString) = "";
+  *tmp = "";
 
   for (i = 0L; i < len; i++) {
     if (cformat[i] != '%') {
       buf[0] = cformat[i];
       buf[1] = '\0';
-      *((*this).tmpString) += buf;
+      *tmp += buf;
     } else if (cformat[i+1] == '%') {
-      *((*this).tmpString) += "%";
+      *tmp += "%";
       i++;
     } else if (cformat[i+1] == 'Y') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.date.year);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '4' && cformat[i+3] == 'Y') {
       ::sprintf(buf, "%04d", (*this).tmpDateTime.date.year);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'm') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.date.mon);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '2' && cformat[i+3] == 'm') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.date.mon);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'd') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.date.mday);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '2' && cformat[i+3] == 'd') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.date.mday);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'H') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.time.hour);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '2' && cformat[i+3] == 'H') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.time.hour);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'M') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.time.min);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '2' && cformat[i+3] == 'M') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.time.min);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'S') {
       ::sprintf(buf, "%d", (*this).tmpDateTime.time.sec);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == '0' && cformat[i+2] == '2' && cformat[i+3] == 'S') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.time.sec);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i += 3;
     } else if (cformat[i+1] == 'a') {
       if (0 <= (*this).tmpDateTime.date.wday && (*this).tmpDateTime.date.wday < 7) {
-        *((*this).tmpString) += AWEEK[(*this).tmpDateTime.date.wday];
+        *tmp += AWEEK[(*this).tmpDateTime.date.wday];
       } else {
-        *((*this).tmpString) += "?";
+        *tmp += "?";
       }
       i++;
     } else if (cformat[i+1] == 'b') {
       if (1 <= (*this).tmpDateTime.date.mon && (*this).tmpDateTime.date.mon <= 12) {
-        *((*this).tmpString) += AMONTH[(*this).tmpDateTime.date.mon - 1];
+        *tmp += AMONTH[(*this).tmpDateTime.date.mon - 1];
       } else {
-        *((*this).tmpString) += "?";
+        *tmp += "?";
       }
       i++;
     } else if (cformat[i+1] == 'y') {
       ::sprintf(buf, "%02d", (*this).tmpDateTime.date.year-1900);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == 'A') {
       if (0 <= (*this).tmpDateTime.date.wday && (*this).tmpDateTime.date.wday < 7) {
-        *((*this).tmpString) += FWEEK[(*this).tmpDateTime.date.wday];
+        *tmp += FWEEK[(*this).tmpDateTime.date.wday];
       } else {
-        *((*this).tmpString) += "?";
+        *tmp += "?";
       }
       i++;
     } else if (cformat[i+1] == 'B') {
       if (1 <= (*this).tmpDateTime.date.mon && (*this).tmpDateTime.date.mon <= 12) {
-        *((*this).tmpString) += FMONTH[(*this).tmpDateTime.date.mon - 1];
+        *tmp += FMONTH[(*this).tmpDateTime.date.mon - 1];
       } else {
-        *((*this).tmpString) += "?";
+        *tmp += "?";
       }
       i++;
     } else if (cformat[i+1] == 'Z') {
       gmtoff = (*this).tmpDateTime.gmtoff;
       if (gmtoff < 0) {
-        *((*this).tmpString) += "-";
+        *tmp += "-";
       } else {
-        *((*this).tmpString) += "+";
+        *tmp += "+";
       }
       ::sprintf(buf, "%02ld", gmtoff/3600);
-      *((*this).tmpString) += buf;
-      *((*this).tmpString) += ":";
+      *tmp += buf;
+      *tmp += ":";
       ::sprintf(buf, "%02ld", gmtoff%3600);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     } else if (cformat[i+1] == 'z') {
       gmtoff = (*this).tmpDateTime.gmtoff;
       if (gmtoff < 0) {
-        *((*this).tmpString) += "-";
+        *tmp += "-";
       } else {
-        *((*this).tmpString) += "+";
+        *tmp += "+";
       }
       ::sprintf(buf, "%02ld", gmtoff/3600);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       sprintf(buf, "%02ld", gmtoff%3600);
-      *((*this).tmpString) += buf;
+      *tmp += buf;
       i++;
     }
   }
 
-  return *((*this).tmpString);
+  return *tmp;
+}
+
+
+/*! Get multi-language date and/or time String by specified message type
+    @param msgid message type
+    @param lang language (ex: "en", "ja", "ko", "zh-cn")
+    @return formatted date and/or time String
+ */
+String &DateTime::getMessage(const char * msgid, const char * lang, const char * charset) const {
+  String *tmp = (*this).tmpStr();
+  const char *langs[] = {"en", "ja", "ko", "zh-cn", "de", "es", "fr", NULL};
+  const char *(*calendar_msg)[][2];
+  const char *wday[7], *wday_long[7];
+  const char *month[12], *month_long[12];
+  String format;
+  int i, detected;
+  const char *cpstr;
+
+  if (msgid == NULL) {
+    *tmp = "";
+    return *tmp;
+  }
+
+  detected = 0;
+  for (i = 0; langs[i] != NULL; i++) {
+    if (!strcasecmp(lang, langs[i])) {
+      detected = 1;
+      lang = langs[i];
+      break;
+    }
+  }
+
+  if (detected == 0) {
+    lang = langs[0]; // default is "en"
+  }
+
+  calendar_msg = &calendar_msg_en;
+  if (lang == langs[1]) {
+    // ja
+    calendar_msg = &calendar_msg_ja;
+  } else if (lang == langs[2]) {
+    // ko
+    calendar_msg = &calendar_msg_ko;
+  } else if (lang == langs[3]) {
+    // zh-cn
+    calendar_msg = &calendar_msg_zh_cn;
+  } else if (lang == langs[4]) {
+    // de
+    calendar_msg = &calendar_msg_de;
+  } else if (lang == langs[5]) {
+    // es
+    calendar_msg = &calendar_msg_es;
+  } else if (lang == langs[6]) {
+    // es
+    calendar_msg = &calendar_msg_fr;
+  }
+
+  format = "";
+  for (i = 0; (cpstr = (*calendar_msg)[i][0]) != (const char *)0; i++) {
+    if (!strcmp(cpstr, msgid)) {
+      format = (*calendar_msg)[i][1];
+      break;
+    }
+  }
+
+  for (i = 0; (cpstr = (*calendar_msg)[i][0]) != (const char *)0; i++) {
+    if (!strncmp(cpstr, "WDAY_", 5)) {
+      if (!strcmp(cpstr, "WDAY_SUN")) {
+        wday[0] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_MON")) {
+        wday[1] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_TUE")) {
+        wday[2] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_WED")) {
+        wday[3] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_THU")) {
+        wday[4] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_FRI")) {
+        wday[5] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_SAT")) {
+        wday[6] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_SUN_LONG")) {
+        wday_long[0] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_MON_LONG")) {
+        wday_long[1] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_TUE_LONG")) {
+        wday_long[2] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_WED_LONG")) {
+        wday_long[3] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_THU_LONG")) {
+        wday_long[4] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_FRI_LONG")) {
+        wday_long[5] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "WDAY_SAT_LONG")) {
+        wday_long[6] = (*calendar_msg)[i][1];
+      }
+    } else if (!strncmp(cpstr, "MONTH", 5)) {
+      if (!strcmp(cpstr, "MONTH1")) {
+        month[0] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH2")) {
+        month[1] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH3")) {
+        month[2] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH4")) {
+        month[3] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH5")) {
+        month[4] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH6")) {
+        month[5] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH7")) {
+        month[6] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH8")) {
+        month[7] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH9")) {
+        month[8] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH10")) {
+        month[9] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH11")) {
+        month[10] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH12")) {
+        month[11] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH1_LONG")) {
+        month_long[0] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH2_LONG")) {
+        month_long[1] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH3_LONG")) {
+        month_long[2] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH4_LONG")) {
+        month_long[3] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH5_LONG")) {
+        month_long[4] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH6_LONG")) {
+        month_long[5] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH7_LONG")) {
+        month_long[6] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH8_LONG")) {
+        month_long[7] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH9_LONG")) {
+        month_long[8] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH10_LONG")) {
+        month_long[9] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH11_LONG")) {
+        month_long[10] = (*calendar_msg)[i][1];
+      } else if (!strcmp(cpstr, "MONTH12_LONG")) {
+        month_long[11] = (*calendar_msg)[i][1];
+      }
+    }
+  }
+
+  format = format.replace("%a", wday[(*this).tmpDateTime.date.wday]);
+  format = format.replace("%A", wday_long[(*this).tmpDateTime.date.wday]);
+  format = format.replace("%b", month[(*this).tmpDateTime.date.mon-1]);
+  format = format.replace("%B", month_long[(*this).tmpDateTime.date.mon-1]);
+
+  *tmp = (*this).toString(format).strconv("UTF-8", charset);
+
+  format.clear();
+  return *tmp;
 }
 
 
@@ -1318,7 +1554,7 @@ int DateTime::compare(const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator == (const DateTime &value) const {
-  return (compare(value) == 0)?1:0;
+  return ((*this).compare(value) == 0)?1:0;
 }
 
 
@@ -1328,7 +1564,7 @@ int DateTime::operator == (const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator != (const DateTime &value) const {
-  return (compare(value) != 0)?1:0;
+  return ((*this).compare(value) != 0)?1:0;
 }
 
 
@@ -1338,7 +1574,7 @@ int DateTime::operator != (const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator <= (const DateTime &value) const {
-  return (compare(value) <= 0)?1:0;
+  return ((*this).compare(value) <= 0)?1:0;
 }
 
 
@@ -1348,7 +1584,7 @@ int DateTime::operator <= (const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator >= (const DateTime &value) const {
-  return (compare(value) >= 0)?1:0;
+  return ((*this).compare(value) >= 0)?1:0;
 }
 
 
@@ -1358,7 +1594,7 @@ int DateTime::operator >= (const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator < (const DateTime &value) const {
-  return (compare(value) < 0)?1:0;
+  return ((*this).compare(value) < 0)?1:0;
 }
 
 
@@ -1368,7 +1604,7 @@ int DateTime::operator < (const DateTime &value) const {
     @retval 0  if not matched
  */
 int DateTime::operator > (const DateTime &value) const {
-  return (compare(value) > 0)?1:0;
+  return ((*this).compare(value) > 0)?1:0;
 }
 
 } // namespace apolloron
