@@ -25,9 +25,10 @@ Keys::Keys() {
   for (i = 0; i < 64; i++) {
     (*this).nElementTotal[i] = 0;
     (*this).nElementStart[i] = -1;
-    (*this).nElementCache[i] = -1;
     (*this).pElement[i] = (Element **)NULL;
   }
+
+  (*this).tmpString = (String *)NULL;
 }
 
 
@@ -42,10 +43,11 @@ Keys::Keys(const Keys &keys) {
   for (i = 0; i < 64; i++) {
     (*this).nElementTotal[i] = 0;
     (*this).nElementStart[i] = -1;
-    (*this).nElementCache[i] = -1;
     (*this).pElement[i] = (Element **)NULL;
   }
 
+  (*this).tmpString = (String *)NULL;
+
   set(keys);
 }
 
@@ -87,6 +89,11 @@ bool Keys::clear() {
     (*this).pElement = NULL;
   }
 
+  if ((*this).tmpString != (String *)NULL) {
+    delete (*this).tmpString;
+    (*this).tmpString = (String *)NULL;
+  }
+
   return true;
 }
 
@@ -109,51 +116,108 @@ String& Keys::operator [] (const char *key) {
 }
 
 
-/*! Get an element that have match key.
+/*! Get value of an element that have match key.
+    @param key  key of object
+    @return Keys object
+ */
+String& Keys::getValue(const String &key) {
+  return (*this).getValue(key.c_str());
+}
+
+
+/*! Get value of an element that have match key.
     @param key  key of object
     @return Keys object
  */
 String& Keys::getValue(const char *key) {
   Element *element;
+  bool found;
+  int start;
 
   element = NULL;
 
-  if (key != NULL) {
-    bool found;
-    int start;
-    found = false;
-    start = b64(key[0]);
-    if (0 <= (*this).nElementCache[start]) {
+  if (key == NULL) {
+    key = "";
+  }
+
+  found = false;
+  start = b64(key[0]);
+  do {
+    if (0 < (*this).nElementTotal[start]) {
       element = (*this).pElement[start][(*this).nElementStart[start]];
-    } else {
-      do {
-        if (0 < (*this).nElementTotal[start]) {
-          element = (*this).pElement[start][(*this).nElementStart[start]];
-          while (element != NULL && element->key != NULL) {
-            int cmp;
-            cmp = strcmp(key, element->key);
-            if (cmp == 0) {
-              // found!
-              found = true;
-              break;
-            } else if (cmp < 0) {
-              // small
-              if (element->small == -1) break;
-              element = (*this).pElement[start][element->small];
-              (*this).nElementCache[start] = element->small;
-            } else {
-              // big
-              if (element->big == -1) break;
-              element = (*this).pElement[start][element->big];
-              (*this).nElementCache[start] = element->big;
-            }
-          }
+      while (element != NULL && element->key != NULL) {
+        int cmp;
+        cmp = strcmp(key, element->key);
+        if (cmp == 0) {
+          // found!
+          found = true;
+          break;
+        } else if (cmp < 0) {
+          // small
+          if (element->small == -1) break;
+          element = (*this).pElement[start][element->small];
+        } else {
+          // big
+          if (element->big == -1) break;
+          element = (*this).pElement[start][element->big];
         }
-      } while (found == false && (*this).addKey(key) == true);
+      }
+    }
+  } while (found == false && (*this).addKey(key) == true);
+
+  return *((*element).value);
+}
+
+
+/*! Get value of an element that have match key. (for multi-threaded use)
+    @param key  key of object
+    @return Keys object
+ */
+const char* Keys::read(const String &key) const {
+  return (*this).read(key.c_str());
+}
+
+
+/*! Get value of an element that have match key. (for multi-threaded use)
+    @param key  key of object
+    @return Keys object
+ */
+const char* Keys::read(const char *key) const {
+  Element *element;
+  bool found;
+  int start;
+
+  element = NULL;
+
+  found = false;
+  start = b64(key[0]);
+
+  if (0 < (*this).nElementTotal[start]) {
+    element = (*this).pElement[start][(*this).nElementStart[start]];
+    while (element != NULL && element->key != NULL) {
+      int cmp;
+      cmp = strcmp(key, element->key);
+      if (cmp == 0) {
+        // found!
+        found = true;
+        break;
+      } else if (cmp < 0) {
+        // small
+        if (element->small == -1) break;
+        element = (*this).pElement[start][element->small];
+      } else {
+        // big
+        if (element->big == -1) break;
+        element = (*this).pElement[start][element->big];
+      }
     }
   }
 
-  return *((*element).value);
+  if (found == false) {
+    return NULL;
+  }
+
+  return (*((*element).value)).c_str();
 }
 
 
@@ -182,29 +246,22 @@ bool Keys::isKey(const char *key) const {
     int start;
     start = b64(key[0]);
     if (0 < (*this).nElementTotal[start]) {
-      if (0 <= (*this).nElementCache[start]) {
-        element = (*this).pElement[start][(*this).nElementStart[start]];
-        if (!strcmp(key, element->key)) {
+      element = (*this).pElement[start][(*this).nElementStart[start]];
+      while (element != NULL && element->key != NULL) {
+        int cmp;
+        cmp = strcmp(key, element->key);
+        if (cmp == 0) {
+          // found!
           found = true;
-        }
-      } else {
-        element = (*this).pElement[start][(*this).nElementStart[start]];
-        while (element != NULL && element->key != NULL) {
-          int cmp;
-          cmp = strcmp(key, element->key);
-          if (cmp == 0) {
-            // found!
-            found = true;
-            break;
-          } else if (cmp < 0) {
-            // small
-            if (element->small == -1) break;
-            element = (*this).pElement[start][element->small];
-          } else {
-            // big
-            if (element->big == -1) break;
-            element = (*this).pElement[start][element->big];
-          }
+          break;
+        } else if (cmp < 0) {
+          // small
+          if (element->small == -1) break;
+          element = (*this).pElement[start][element->small];
+        } else {
+          // big
+          if (element->big == -1) break;
+          element = (*this).pElement[start][element->big];
         }
       }
     }
@@ -309,116 +366,66 @@ Element* Keys::array(long index) const {
       }
     }
 
+    if (0 <= start) {
+      long internal_index = index - (total - (*this).nElementTotal[start]);
+      if (0 <= internal_index && internal_index <= (*this).nElementTotal[start]) {
+        element = (*this).pElement[start][internal_index];
+      }
+    }
+/*
     if (0 <= start) {
       long internal_index;
       long *stack;
+      char *stack_info; // -1: small, 1: big
       long stacknum;
-      enum {UP, DOWN} dest;
-      long count;
 
       internal_index = index - (total - (*this).nElementTotal[start]);
-      stack = new long[(*this).nElementTotal[start]];
-      for (stacknum = 0; stacknum < (*this).nElementTotal[start]; stacknum++) {
-        stack[stacknum] = -1;
-      }
+      stack = new long[(*this).nElementTotal[start] + 1];
+      stack_info = new char[(*this).nElementTotal[start] + 1];
       stacknum = 0;
       stack[stacknum] = (*this).nElementStart[start];
+      stack_info[stacknum] = -1;
       stacknum++;
-      dest = DOWN;
+      stack[stacknum] = -1;
 
-      if (internal_index <= (*this).nElementTotal[start]) {
-        count = 0;
+      if (0 <= internal_index && internal_index <= (*this).nElementTotal[start]) {
+        long count = 0;
         while (count < internal_index) {
-          if (dest == DOWN) {
-            // stack down
-            if (0 <= (*this).pElement[start][stack[stacknum - 1]]->small) {
-              stack[stacknum] = (*this).pElement[start][stack[stacknum - 1]]->small;
+          if (stack_info[stacknum - 1] == -1) {
+            long small = (*this).pElement[start][stack[stacknum - 1]]->small;
+            stack_info[stacknum - 1] = 1;
+            if (0 <= small) {
+              stack[stacknum] = small;
+              stack_info[stacknum] = -1;
               stacknum++;
-            } else {
-              if (1 < stacknum) {
-                count++;
-                if (internal_index <= count) {
-                  break;
-                }
-              }
-              if (0 <= (*this).pElement[start][stack[stacknum - 1]]->big) {
-                stack[stacknum] = (*this).pElement[start][stack[stacknum - 1]]->big;
-                stacknum++;
-              } else {
-                stacknum--;
-                dest = UP;
-              }
+              stack[stacknum] = -1;
+              count++;
+            }
+          } else if (stack_info[stacknum - 1] == 1) {
+            long big = (*this).pElement[start][stack[stacknum - 1]]->big;
+            stack_info[stacknum - 1] = 0;
+            if (0 <= big) {
+              stack[stacknum] = big;
+              stack_info[stacknum] = -1;
+              stacknum++;
+              stack[stacknum] = -1;
+              count++;
             }
           } else {
-            // stack up
-            while (0 < stacknum) {
-              long  big;
-              big = (*this).pElement[start][stack[stacknum - 1]]->big;
-              if (!(0 <= big && big == stack[stacknum])) {
-                count++;
-                if (internal_index <= count) {
-                  break;
-                }
-              }
-              if (0 <= big && big != stack[stacknum]) {
-                stack[stacknum] = big;
-                stacknum++;
-                dest = DOWN;
-                break;
-              }
-              stacknum--;
-            }
-          }
-        }
-      } else {
-        count = (*this).nElementTotal[start] - 1;
-        while (internal_index <= count) {
-          if (dest == DOWN) {
-            // stack down
-            if (0 <= (*this).pElement[start][stack[stacknum - 1]]->big) {
-              stack[stacknum] = (*this).pElement[start][stack[stacknum - 1]]->big;
-              stacknum++;
-            } else {
-              if (1 < stacknum) {
-                count--;
-                if (count < internal_index) {
-                  break;
-                }
-              }
-              if (0 <= (*this).pElement[start][stack[stacknum - 1]]->small) {
-                stack[stacknum] = (*this).pElement[start][stack[stacknum - 1]]->small;
-                stacknum++;
-              } else {
-                stacknum--;
-                dest = UP;
-              }
-            }
-          } else {
-            // stack up
-            while (0 < stacknum) {
-              long  small;
-              small = (*this).pElement[start][stack[stacknum - 1]]->small;
-              if (!(0 <= small && small == stack[stacknum])) {
-                count--;
-                if (count < internal_index) {
-                  break;
-                }
-              }
-              if (0 <= small && small != stack[stacknum]) {
-                stack[stacknum] = small;
-                stacknum++;
-                dest = DOWN;
-                break;
-              }
-              stacknum--;
+            stack[stacknum] = -1;
+            if (stacknum <= 1) {
+              break;
             }
+            stacknum--;
           }
         }
       }
 
       element = (*this).pElement[start][stack[stacknum - 1]];
+      delete [] stack_info;
       delete [] stack;
     }
+*/
   }
 
   return element;
@@ -530,28 +537,27 @@ bool Keys::addKey(const char *key, const char *value) {
                 if (cmp < 0) {
                   (*this).pElement[start][(*this).nElementTotal[start]]->big = parent_element->small;
                   (*this).pElement[start][(*this).nElementTotal[start]]->small = target_element->small;
+                  parent_element->small = (*this).nElementTotal[start];
                   target_element->small = -1;
                 } else {
                   (*this).pElement[start][(*this).nElementTotal[start]]->big = parent_element->small;
                   (*this).pElement[start][(*this).nElementTotal[start]]->big = target_element->big;
+                  parent_element->small = (*this).nElementTotal[start];
                   target_element->big = -1;
                 }
               } else {
                 if (cmp < 0) {
                   (*this).pElement[start][(*this).nElementTotal[start]]->small = parent_element->big;
                   (*this).pElement[start][(*this).nElementTotal[start]]->small = target_element->small;
+                  parent_element->big = (*this).nElementTotal[start];
                   target_element->small = -1;
                 } else {
                   (*this).pElement[start][(*this).nElementTotal[start]]->small = parent_element->big;
                   (*this).pElement[start][(*this).nElementTotal[start]]->big = target_element->big;
+                  parent_element->big = (*this).nElementTotal[start];
                   target_element->big = -1;
                 }
               }
-              if (cmp < 0) {
-                parent_element->small = (*this).nElementTotal[start];
-              } else {
-                parent_element->big = (*this).nElementTotal[start];
-              }
               (*this).nElementTotal[start]++;
               break;
             }
@@ -699,7 +705,90 @@ bool Keys::delKey(const char *key) {
         target_element = (*this).pElement[start][target];
       }
     }
-    (*this).nElementCache[start] = -1;
+  }
+
+  return true;
+}
+
+
+/*! Convert Keys data to serialized string.
+    @param void
+    @return Serialized string
+ */
+String &Keys::toString() {
+  long m, i;
+  String tmp_str1, tmp_str2;
+
+  if ((*this).tmpString == (String *)NULL) {
+    (*this).tmpString = new String;
+  }
+  *((*this).tmpString) = "";
+
+  m = (*this).max();
+  (*((*this).tmpString)).sprintf("# %ld key(s)\n", m);
+  for (i = 0; i < m; i++) {
+    *((*this).tmpString) += (*this).key(i);
+    tmp_str1 = (*this).value(i).trim();
+    tmp_str2 = (*this).value(i).escapeQuote();
+    if (tmp_str1 == tmp_str2) {
+      *((*this).tmpString) += "=";
+      *((*this).tmpString) += tmp_str1;
+      *((*this).tmpString) += "\n";
+    } else {
+      *((*this).tmpString) += "=\"";
+      *((*this).tmpString) += tmp_str2;
+      *((*this).tmpString) += "\"\n";
+    }
+  }
+
+  tmp_str1.clear();
+  tmp_str2.clear();
+
+  return *((*this).tmpString);
+}
+
+
+/*! Set Keys structure by serialized string.
+    @param str    String to set.
+    @retval true  success
+    @retval false failure
+ */
+bool Keys::setString(const String &str) {
+  String line, key, value, tmp_str;
+  long len, i, j, k;
+
+  (*this).clear();
+
+  len = str.len();
+  if (len < 0L) {
+    return false;
+  }
+
+  for (i = 0L; i < len; i++) {
+    for (j = 0L; i + j < len; j++) {
+      if (str[i + j] == '\n') {
+        break;
+      }
+    }
+    if (0L < j) {
+      line = str.mid(i, j).trim();
+      if (0L < line.len() && line[0] != '#' && 0 < (k = line.searchChar('='))) {
+        String p, p1;
+        p = line;
+        p1 = p.left(k).trim();
+        key = p1;
+        p1 = p.mid(k+1).trim();
+        value = p1;
+        if (2L <= value.len() && value[0] == '"' && value[value.len()-1] == '"') {
+          tmp_str = value.mid(1, value.len()-2);
+          value = tmp_str.unescapeQuote();
+          tmp_str.clear();
+        }
+        (*this).addKey(key, value);
+      }
+      line.clear();
+      i += j;
+    }
   }
 
   return true;
