@@ -254,8 +254,8 @@ bool Socket::connect(const char *host, const char *port, bool ssl) {
   }
 
   // connect
-  ::setsockopt((*this).dstSocket, SOL_SOCKET, SO_SNDTIMEO, &t_timeout, sizeof(struct timeval));
-  ::setsockopt((*this).dstSocket, SOL_SOCKET, SO_RCVTIMEO, &t_timeout, sizeof(struct timeval));
+  ::setsockopt((*this).dstSocket, SOL_SOCKET, SO_SNDTIMEO, &timeout, sizeof(struct timeval));
+  ::setsockopt((*this).dstSocket, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(struct timeval));
 
   if (::connect((*this).dstSocket, (struct sockaddr *) &dstAddr, sizeof(dstAddr)) < 0) {
     (*this).nErrno = errno;
