@@ -1638,8 +1638,8 @@ char* sjis_to_jis(const char *str) {
     i = 0;
     j = 0;
     while (i < length) {
-      a0 = (unsigned char)str[i++];
-      if (0x81 < (unsigned char)str[i] && (unsigned char)str[i] <= 0x9F) {
+      a0 = (unsigned char)str[i];
+      if (0x81 <= (unsigned char)str[i] && (unsigned char)str[i] <= 0x9F) {
         // Kanji
         // From division 1 to division 62
         a1 = str[i+1];
@@ -1665,7 +1665,7 @@ char* sjis_to_jis(const char *str) {
         }
         buf[j++] = a0;
         buf[j] = a1;
-        i++;
+        i += 2;
       } else if (0xE0 <= (unsigned char)str[i] && (unsigned char)str[i] <= 0xF9) {
         // From division 63 to division 94
         a0 = (str[i] - 0xE0) << 1;
@@ -1689,7 +1689,7 @@ char* sjis_to_jis(const char *str) {
         }
         buf[j++] = a0;
         buf[j] = a1;
-        i++;
+        i += 2;
       } else if (0xFA <= (unsigned char)str[i] && (unsigned char)str[i] <= 0xFC) {
         // From division 115 to division 119 to CP50220
         unsigned char s1;
@@ -1716,7 +1716,7 @@ char* sjis_to_jis(const char *str) {
         }
         buf[j++] = (linear - 0x819E) / 94;
         buf[j] = (linear % 94) + 0x21;
-        i++;
+        i += 2;
       } else {
         // Non-Kanji
         if (jiskanji) {
@@ -1726,9 +1726,9 @@ char* sjis_to_jis(const char *str) {
           jiskanji = 0;
         }
         buf[j] = str[i];
+        i++;
       }
       j++;
-      i++;
     }
 
     if (jiskanji) {
