@@ -22,6 +22,8 @@ int test7();
 int test8();
 int test9();
 int test10();
+int test11();
+int test12();
 int example1();
 int example2();
 
@@ -101,13 +103,27 @@ int main(int argc, char *argv[]) {
   }
   fprintf(stderr, "OK\n");
 
-  fprintf(stderr, "Starting Test10 DateTime Class ... ");
+  fprintf(stderr, "Starting Test10 Sheet Class ... ");
   status = test10();
   if (status != 0) {
     return -1;
   }
   fprintf(stderr, "OK\n");
 
+  fprintf(stderr, "Starting Test11 DateTime Class ... ");
+  status = test11();
+  if (status != 0) {
+    return -1;
+  }
+  fprintf(stderr, "OK\n");
+
+  fprintf(stderr, "Starting Test12 MIMEHeader Class ... ");
+  status = test12();
+  if (status != 0) {
+    return -1;
+  }
+  fprintf(stderr, "OK\n");
+
 //  example1();
 //  example2();
 
@@ -252,7 +268,8 @@ int test2() {
 int test3() {
   // Declare Strings
   String str_a, str_b, str_c, str_d, str_e, str_f, str_g, str_h, str_i, str_j,
-         str_k, str_l, str_m, str_n, str_o;
+         str_k, str_l, str_m, str_n, str_o, str_p, str_q, str_r;
+  String name, user, domain;
 
   // Set Values
   str_a = "This is a pen.";                          // Set String
@@ -278,6 +295,12 @@ int test3() {
   str_o = str_o.cramMd5("account",
           "PEMzN0QxRDA5QkREMURBRjQ5NjI5REFCQzE0QzdCMDJBQHZpc2lvbj4=");
 #endif
+#if __SHA1 == 1
+  str_p = "Hello~";
+  str_p = str_p.sha1();
+  str_q = "what do ya want for nothing?"; // Data
+  str_q = str_q.hmacSha1("Jefe"); // Key="Jefe"
+#endif
 
   // Expected Results
   // str_a = This is a pen.
@@ -295,6 +318,8 @@ int test3() {
   // str_m = abc\r\ndef\r
   // str_n = abcABCdefABC
   // str_o = YWNjb3VudCA4MTk3M2Q3NTllYWRjMjBjYTNhMGRmZDgxMDQwMDg1ZQ==
+  // str_p = 70b6fef3be85ab78eae4b9ba38c74f5b07252341
+  // str_q = effcdf6ae5eb2fa2d27416d5f184df9c259a7c79
 
   if (strcmp(str_a.c_str(), "This is a pen.") != 0) {
     fprintf(stderr, "Error: Test3 #1\n");
@@ -365,6 +390,48 @@ int test3() {
   }
 #endif
 
+#if __SHA1 == 1
+  if (strcmp(str_p.c_str(), "70b6fef3be85ab78eae4b9ba38c74f5b07252341") != 0) {
+    fprintf(stderr, "Error: Test3 #14\n");
+    return -1;
+  }
+
+  if (strcmp(str_q.c_str(), "effcdf6ae5eb2fa2d27416d5f184df9c259a7c79") != 0) {
+    fprintf(stderr, "Error: Test3 #15\n");
+    return -1;
+  }
+#endif
+
+  str_r = "=?ISO-2022-JP?B?GyRCJWYhPCU2GyhCMQ==?= <user3@example.com>";
+  str_r.parseEmail(name, user, domain);
+  if (strcmp(name.c_str(), "=?ISO-2022-JP?B?GyRCJWYhPCU2GyhCMQ==?=") != 0) {
+    fprintf(stderr, "Error: Test3 #16\n");
+    return -1;
+  }
+  if (strcmp(user.c_str(), "user3") != 0) {
+    fprintf(stderr, "Error: Test3 #17\n");
+    return -1;
+  }
+  if (strcmp(domain.c_str(), "example.com") != 0) {
+    fprintf(stderr, "Error: Test3 #18\n");
+    return -1;
+  }
+
+  str_r = " \"Nakamura, Tarou\" <nakamura@example.com > ";
+  str_r.parseEmail(name, user, domain);
+  if (strcmp(name.c_str(), "Nakamura, Tarou") != 0) {
+    fprintf(stderr, "Error: Test3 #19\n");
+    return -1;
+  }
+  if (strcmp(user.c_str(), "nakamura") != 0) {
+    fprintf(stderr, "Error: Test3 #20\n");
+    return -1;
+  }
+  if (strcmp(domain.c_str(), "example.com") != 0) {
+    fprintf(stderr, "Error: Test3 #21\n");
+    return -1;
+  }
+
   // Clear Allocated Memories (option)
   str_a.clear();
   str_b.clear();
@@ -384,6 +451,11 @@ int test3() {
 #if __MD5 == 1
   str_o.clear();
 #endif
+#if __SHA1 == 1
+  str_p.clear();
+  str_q.clear();
+#endif
+  str_r.clear();
 
   return 0;
 }
@@ -617,33 +689,33 @@ int test7() {
 int test8() {
   // Declare Strings
   String str_a, str_b, str_c;
-  List list;
+  List list1, list2;
 
   // Set Values
   str_a = "Tarou";                                // Set String
   str_b = "is";                                   // Set String
   str_c = "friend";                               // Set String
-  list.add(str_a);                                // Append String Into List
-  list.add(str_b);                                // Append String Into List
-  list.add(str_c);                                // Append String Into List
+  list1.add(str_a);                               // Append String Into List
+  list1.add(str_b);                               // Append String Into List
+  list1.add(str_c);                               // Append String Into List
 
   // Insert Into List
-  list.insert(2, "my");                           // Insert Before 3rd Element
+  list1.insert(2, "my");                          // Insert Before 3rd Element
 
   // Delete From List
-  list.remove(1);                                 // Delete 2nd Element
+  list1.remove(1);                                // Delete 2nd Element
 
   // Replace List Element
-  list[0] += "'s";                                // Replace 1st Element String
+  list1[0] += "'s";                               // Replace 1st Element String
 
   // Expected Results
   // str_a = Tarou
   // str_b = is
   // str_c = friend
-  // list.max() = 3
-  // list[0] = Tarou's
-  // list[1] = my
-  // list[2] = friend
+  // list1.max() = 3
+  // list1[0] = Tarou's
+  // list1[1] = my
+  // list1[2] = friend
 
   if (strcmp(str_a.c_str(), "Tarou") != 0) {
     fprintf(stderr, "Error: Test8 #1\n");
@@ -660,31 +732,67 @@ int test8() {
     return -1;
   }
 
-  if (list.max() != 3) {
+  if (list1.max() != 3) {
     fprintf(stderr, "Error: Test8 #4\n");
     return -1;
   }
 
-  if (strcmp(list[0].c_str(), "Tarou's") != 0) {
+  if (strcmp(list1[0].c_str(), "Tarou's") != 0) {
     fprintf(stderr, "Error: Test8 #5\n");
     return -1;
   }
 
-  if (strcmp(list[1].c_str(), "my") != 0) {
+  if (strcmp(list1[1].c_str(), "my") != 0) {
     fprintf(stderr, "Error: Test8 #6\n");
     return -1;
   }
 
-  if (strcmp(list[2].c_str(), "friend") != 0) {
+  if (strcmp(list1[2].c_str(), "friend") != 0) {
     fprintf(stderr, "Error: Test8 #7\n");
     return -1;
   }
 
+  list2.setEmails("user1@example.com, <user2@example.com> , \r\n"
+        "\t=?ISO-2022-JP?B?GyRCJWYhPCU2GyhCMQ==?= <user3@example.com>,"
+	"    a: =?UTF-8?B?7Lus65+s?= <kara@example.com>,\r\n"
+        "    \"Nakamura, Tarou\" <nakamura@example.com> ;");
+
+  if (list2.max() != 5) {
+    fprintf(stderr, "Error: Test8 #8\n");
+    return -1;
+  }
+
+  if (list2[0] != "user1@example.com") {
+    fprintf(stderr, "Error: Test8 #9\n");
+    return -1;
+  }
+
+  if (list2[1] != "<user2@example.com>") {
+    fprintf(stderr, "Error: Test8 #10\n");
+    return -1;
+  }
+
+  if (list2[2] != "=?ISO-2022-JP?B?GyRCJWYhPCU2GyhCMQ==?= <user3@example.com>") {
+    fprintf(stderr, "Error: Test8 #11\n");
+    return -1;
+  }
+
+  if (list2[3] != "=?UTF-8?B?7Lus65+s?= <kara@example.com>") {
+    fprintf(stderr, "Error: Test8 #12\n");
+    return -1;
+  }
+
+  if (list2[4] != "\"Nakamura, Tarou\" <nakamura@example.com>") {
+    fprintf(stderr, "Error: Test8 #13\n");
+    return -1;
+  }
+
   // Clear Allocated Memories (option)
   str_a.clear();
   str_b.clear();
   str_c.clear();
-  list.clear();
+  list1.clear();
+  list2.clear();
 
   return 0;
 }
@@ -771,7 +879,7 @@ int test9() {
   }
 
   if (strcmp(keys["age"].c_str(), "5") != 0) {
-    fprintf(stderr, "Error: Test8 #11\n");
+    fprintf(stderr, "Error: Test9 #11\n");
     return -1;
   }
 
@@ -783,14 +891,116 @@ int test9() {
 }
 
 
-/*! Test10  DateTime Class
+/*! Test10  Sheet Class
     @param  void
     @retval 0  success
     @retval -1 failure
  */
 int test10() {
+  // Declare Sheet
+  Sheet sheet1, sheet2;
+
+  // Set Values
+  sheet1.addCol("COL1");
+  sheet1.addCol("COL2");
+  sheet1.addRow();
+  sheet1.setValue("COL1", 0, "abc");
+  sheet1.setValue("COL2", 0, "123");
+  sheet1.addRow();
+  sheet1.setValue("COL1", 1, "A");
+  sheet1.setValue("COL2", 1, "1");
+  sheet1.delRow(1);
+  sheet1.addRow();
+  sheet1.setValue("COL1", 1, "b\"cd");
+  sheet1.setValue("COL2", 1, "345");
+  sheet1.insertRow(1);
+  sheet1.setValue("COL1", 1, "xyz");
+  sheet1.setValue("COL2", 1, "987");
+
+  // Expected Results
+  // sheet1(COL1, 0) == "abc"
+  // sheet1(COL2, 0) == "123"
+  // sheet1(COL1, 1) == "xyz"
+  // sheet1(COL2, 1) == "987"
+  // sheet1(COL1, 2) == "b\"cd"
+  // sheet1(COL2, 2) == "345"
+
+  if (sheet1.getValue("COL1", 0) != "abc") {
+    fprintf(stderr, "Error: Test10 #1\n");
+    return -1;
+  }
+
+  if (sheet1.getValue("COL2", 0) != "123") {
+    fprintf(stderr, "Error: Test10 #2\n");
+    return -1;
+  }
+
+  if (sheet1.getValue("COL1", 1) != "xyz") {
+    fprintf(stderr, "Error: Test10 #3\n");
+    return -1;
+  }
+
+  if (sheet1.getValue("COL2", 1) != "987") {
+    fprintf(stderr, "Error: Test10 #4\n");
+    return -1;
+  }
+
+  if (sheet1.getValue("COL1", 2) != "b\"cd") {
+    fprintf(stderr, "Error: Test10 #5\n");
+    return -1;
+  }
+
+  if (sheet1.getValue("COL2", 2) != "345") {
+    fprintf(stderr, "Error: Test10 #6\n");
+    return -1;
+  }
+
+  sheet1.setValue("COL2", 2, "0456");
+
+  if (sheet1.getValue("COL2", 2) != "0456") {
+    fprintf(stderr, "Error: Test10 #7\n");
+    return -1;
+  }
+
+  sheet1.sort("COL1");
+
+  if (sheet1.getValue("COL2", 2) != "987") {
+    fprintf(stderr, "Error: Test10 #8\n");
+    return -1;
+  }
+
+  sheet1.sortnumr("COL2");
+  if (sheet1.getValue("COL2", 2) != "123") {
+    fprintf(stderr, "Error: Test10 #9\n");
+    return -1;
+  }
+
+  if (sheet1.getCSV() != "COL1,COL2\nxyz,987\n\"b\"\"cd\",\"0456\"\nabc,123\n") {
+    fprintf(stderr, "Error: Test10 #10\n");
+    return -1;
+  }
+
+  sheet2.setCSV(sheet1.getCSV());
+  if (sheet1 != sheet2) {
+    fprintf(stderr, "Error: Test10 #11\n");
+    return -1;
+  }
+
+  // Clear Allocated Memories (option)
+  sheet1.clear();
+
+  return 0;
+}
+
+
+/*! Test11  DateTime Class
+    @param  void
+    @retval 0  success
+    @retval -1 failure
+ */
+int test11() {
   // Declare Strings
-  String str_a, str_b, str_c, str_d;
+  String str_a, str_b, str_c, str_d, str_e;
   DateTime datetime1, datetime2, datetime3;
 
   // Set Values
@@ -802,30 +1012,37 @@ int test10() {
   datetime3 = datetime2;
   datetime3.adjustDate(10);
   str_d = datetime3.toString("%04Y-%02m-%02dT%02H:%02M:%02S%Z (%A)");
+  str_e = datetime3.getMessage("DATETIME_WDAY_SEC", "en");
 
   // Expected Results
   // str_a = Wed, 26 Dec 2007 14:15:19 +0900 (JST)
   // str_b = 2007/12/26 14:15:19 +09:00
   // str_c = Wed, 26 Dec 2007 14:15:20 +0900
   // str_d = 2008-01-05T14:15:20+09:00 (Saturday)
+  // str_e = 14:15:20 Sat January 5, 2008
 
   if (strcmp(str_b.c_str(), "2007/12/26 14:15:19 +09:00") != 0) {
-    fprintf(stderr, "Error: Test10 #1\n");
+    fprintf(stderr, "Error: Test11 #1\n");
     return -1;
   }
 
   if (strcmp(str_c.c_str(), "Wed, 26 Dec 2007 14:15:20 +0900") != 0) {
-    fprintf(stderr, "Error: Test10 #2\n");
+    fprintf(stderr, "Error: Test11 #2\n");
     return -1;
   }
 
   if (datetime1 >= datetime2) {
-    fprintf(stderr, "Error: Test10 #3\n");
+    fprintf(stderr, "Error: Test11 #3\n");
     return -1;
   }
 
   if (strcmp(str_d.c_str(), "2008-01-05T14:15:20+09:00 (Saturday)") != 0) {
-    fprintf(stderr, "Error: Test10 #4\n");
+    fprintf(stderr, "Error: Test11 #4\n");
+    return -1;
+  }
+
+  if (strcmp(str_e.c_str(), "14:15:20 Sat January 5, 2008") != 0) {
+    fprintf(stderr, "Error: Test12 #4\n");
     return -1;
   }
 
@@ -834,6 +1051,7 @@ int test10() {
   str_b.clear();
   str_c.clear();
   str_d.clear();
+  str_e.clear();
   datetime1.clear();
   datetime2.clear();
   datetime3.clear();
@@ -842,6 +1060,60 @@ int test10() {
 }
 
 
+/*! Test12  MIMEHeader Class
+    @param  void
+    @retval 0  success
+    @retval -1 failure
+ */
+int test12() {
+  // Declare Strings
+  String str_a;
+  MIMEHeader header;
+
+  // Set Values
+  str_a = "Content-Type: text/plain; charset=UTF-8\r\n"
+          "MIME-Version: 1.0\r\n"
+          "Spam-Check: 1\r\n"
+          " checked\r\n";
+
+  // Expected Results
+  // header.getValue("CONTENT-TYPE") == "text/plain; charset=UTF-8"
+  // header.getValue("mime-version") == "1.0"
+  // header.getValue("Spam-Check") == "1\r\n checked"
+
+  header.set(str_a);
+
+  if (strcmp(header.getValue("CONTENT-TYPE").c_str(), "text/plain; charset=UTF-8") != 0) {
+    fprintf(stderr, "Error: Test12 #1\n");
+    return -1;
+  }
+
+  if (strcmp(header.getValue("mime-version").c_str(), "1.0") != 0) {
+    fprintf(stderr, "Error: Test12 #2\n");
+    return -1;
+  }
+
+  if (strcmp(header.getValue("Spam-Check").c_str(), "1\r\n checked") != 0) {
+    fprintf(stderr, "Error: Test12 #3\n");
+    return -1;
+  }
+
+  if (strcmp(header.toString(), "Content-Type: text/plain; charset=UTF-8\r\n"
+                                "MIME-Version: 1.0\r\n"
+                                "Spam-Check: 1\r\n"
+                                " checked\r\n") != 0) {
+    fprintf(stderr, "Error: Test12 #4\n");
+    return -1;
+  }
+
+  // Clear Allocated Memories (option)
+  str_a.clear();
+  header.clear();
+
+  return 0;
+}
+
+
 /*! Example1  Socket Class
     @param  void
     @retval 0  success
